package at.bestsolution.rsd.rt.client.wildfly;

import java.util.List;
import java.util.Optional;
import java.util.ServiceLoader;

import org.jboss.ejb.client.EJBClientInterceptor;
import org.jboss.ejb.client.EJBClientInvocationContext;

import at.bestsolution.rsd.rt.ServiceProviderSystem;
import at.bestsolution.rsd.rt.client.ejb.EJBCallInterceptor;
import at.bestsolution.rsd.rt.client.ejb.EJBResultInterceptor;

public class WildflyEJBClientInterceptor implements EJBClientInterceptor {
	private volatile List<EJBCallInterceptor> callInterceptors;
	private volatile List<EJBResultInterceptor> resultInterceptors;

	private List<EJBCallInterceptor> lookupCallInterceptors() {
		
		List<EJBCallInterceptor> local = this.callInterceptors;

		if (local == null) {
			synchronized (this) {
				local = this.callInterceptors;
				if (local == null) {
					ServiceProviderSystem serviceProviderSystem = ServiceLoader.load(ServiceProviderSystem.class).iterator().next();
					this.callInterceptors = local = serviceProviderSystem.lookupServices(EJBCallInterceptor.class);
				}
			}
		}

		return local;
	}

	private List<EJBResultInterceptor> lookupResultInterceptors() {
		List<EJBResultInterceptor> local = this.resultInterceptors;

		if (local == null) {
			synchronized (this) {
				local = this.resultInterceptors;
				if (local == null) {
					ServiceProviderSystem serviceProviderSystem = ServiceLoader.load(ServiceProviderSystem.class).iterator().next();
					this.resultInterceptors = local = serviceProviderSystem.lookupServices(EJBResultInterceptor.class);
				}
			}
		}

		return local;
	}

	@Override
	public void handleInvocation(EJBClientInvocationContext context) throws Exception {
		if (!lookupCallInterceptors().isEmpty()) {
			EJBCallContext ctx = new EJBCallContext(context);
			for (EJBCallInterceptor interceptor : callInterceptors) {
				boolean status = interceptor.preInvokation(ctx);
				if (!status) {
					throw new IllegalStateException(String.format("Invocation aborted by %s", interceptor.name()));
				}
			}
		}

		try {
			context.sendRequest();
		} finally {
			if (!lookupResultInterceptors().isEmpty()) {
				EJBCallContext ctx = new EJBCallContext(context);
				for (EJBCallInterceptor interceptor : callInterceptors) {
					interceptor.postInvokation(ctx);
				}
			}
		}
	}

	@Override
	public Object handleInvocationResult(EJBClientInvocationContext context) throws Exception {
		Object result = context.getResult();
		if (!resultInterceptors.isEmpty()) {
			EJBResultContext ctx = new EJBResultContext(context);
			for (EJBResultInterceptor interceptor : resultInterceptors) {
				result = interceptor.process(ctx, result);
			}
		}
		return result;
	}

	static class EJBCallContext implements EJBCallInterceptor.CallContext {
		private final EJBClientInvocationContext context;

		public EJBCallContext(EJBClientInvocationContext context) {
			this.context = context;
		}

		@Override
		public void setProperty(String key, Object value) {
			this.context.getContextData().put(key, value);
		}

		@Override
		public String getMethod() {
			return this.context.getInvokedMethod().getDeclaringClass().getName() + "#"
					+ this.context.getInvokedMethod().getName();
		}
	}

	static class EJBResultContext implements EJBResultInterceptor.ResultContext {
		private final EJBClientInvocationContext context;

		public EJBResultContext(EJBClientInvocationContext context) {
			this.context = context;
		}

		@Override
		public Optional<Object> getProperty(String key) {
			return Optional.ofNullable(this.context.getContextData().get(key));
		}

		@Override
		public String getMethod() {
			return this.context.getInvokedMethod().getDeclaringClass().getName() + "#"
					+ this.context.getInvokedMethod().getName();
		}
	}
}
