package at.bestsolution.rsd.rt.client.wildfly;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Optional;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.eclipse.microprofile.config.ConfigProvider;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import at.bestsolution.rsd.rt.client.ServiceLookup;
import at.bestsolution.rsd.rt.client.ejb.EJBServiceDescription;

@Component(property = ServiceLookup.SERVICE_TYPE + ":String=ejb")
public class WildflyServiceLookup implements ServiceLookup {
	private Map<Class<?>, String> map = new HashMap<>();
	
	private volatile Context context;
	
	private Context getContext() {
		Context localContext = context;
		
		if( localContext == null ) {
			synchronized (this) {
				localContext = context;
				if( localContext == null ) {
					final HashMap<String, String> jndiProperties = new HashMap<>();
					jndiProperties.put(Context.INITIAL_CONTEXT_FACTORY, "org.wildfly.naming.client.WildFlyInitialContextFactory");
					jndiProperties.put(Context.PROVIDER_URL, ConfigProvider.getConfig().getOptionalValue("rsd.wildfly.url", String.class).orElse("remote+http://localhost:8080"));
					jndiProperties.put(Context.URL_PKG_PREFIXES, "org.jboss.ejb.client.naming" );
					Optional<String> principal = ConfigProvider.getConfig().getOptionalValue("rsd.wildfly.security.principal", String.class);
					Optional<String> credentials = ConfigProvider.getConfig().getOptionalValue("rsd.wildfly.security.credentials", String.class);
					if( principal.isPresent() ) {
						jndiProperties.put(Context.SECURITY_PRINCIPAL, principal.get());	
					}
					if( credentials.isPresent() ) {
						jndiProperties.put(Context.SECURITY_CREDENTIALS, credentials.get());
					}
					
					try {
						context = localContext = new InitialContext(new Hashtable<>(jndiProperties));
					} catch (Exception e) {
						throw new IllegalStateException(e);
					}
				}
			}
		}
		
		return localContext;
	}
	
	@Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	public void registerEJBService(EJBServiceDescription ejbService) {
		map.put(ejbService.type(), ejbService.lookupName());
	}
	
	public void unregisterEJBService(EJBServiceDescription ejbService) {
		map.remove(ejbService.type());
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public <S> Optional<S> lookupService(Class<S> clazz) {
		String lookup = map.get(clazz);
		if( lookup == null ) {
			return Optional.empty();
		} else {
			try {
				return Optional.ofNullable((S)getContext().lookup(lookup));
			} catch (NamingException e) {
				throw new IllegalStateException(e);
			}
		}
	}

}
